unit REMain;

interface

uses
  SysUtils, Windows, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, StdCtrls, Buttons, ExtCtrls, Menus, ComCtrls, ClipBrd,
  ToolWin, ActnList, ImgList, QuickPdf, QPDFRender, jpeg, TeeProcs, TeEngine,
  Chart;

type
  TMainForm = class(TForm)
    MainMenu: TMainMenu;
    FileNewItem: TMenuItem;
    FileOpenItem: TMenuItem;
    FileSaveItem: TMenuItem;
    FileSaveAsItem: TMenuItem;
    FilePrintItem: TMenuItem;
    FileExitItem: TMenuItem;
    EditUndoItem: TMenuItem;
    EditCutItem: TMenuItem;
    EditCopyItem: TMenuItem;
    EditPasteItem: TMenuItem;
    OpenDialog: TOpenDialog;
    SaveDialog: TSaveDialog;
    PrintDialog: TPrintDialog;
    Ruler: TPanel;
    FontDialog1: TFontDialog;
    FirstInd: TLabel;
    LeftInd: TLabel;
    RulerLine: TBevel;
    RightInd: TLabel;
    N5: TMenuItem;
    miEditFont: TMenuItem;
    Editor: TRichEdit;
    StatusBar: TStatusBar;
    StandardToolBar: TToolBar;
    OpenButton: TToolButton;
    SaveButton: TToolButton;
    PrintButton: TToolButton;
    ToolButton5: TToolButton;
    UndoButton: TToolButton;
    CutButton: TToolButton;
    CopyButton: TToolButton;
    PasteButton: TToolButton;
    ToolButton10: TToolButton;
    FontName: TComboBox;
    FontSize: TEdit;
    ToolButton11: TToolButton;
    UpDown1: TUpDown;
    BoldButton: TToolButton;
    ItalicButton: TToolButton;
    UnderlineButton: TToolButton;
    ToolButton16: TToolButton;
    LeftAlign: TToolButton;
    CenterAlign: TToolButton;
    RightAlign: TToolButton;
    ToolButton20: TToolButton;
    BulletsButton: TToolButton;
    ToolbarImages: TImageList;
    ActionList1: TActionList;
    FileNewCmd: TAction;
    FileOpenCmd: TAction;
    FileSaveCmd: TAction;
    FilePrintCmd: TAction;
    FileExitCmd: TAction;
    ToolButton1: TToolButton;
    ToolButton2: TToolButton;
    Bevel1: TBevel;
    EditCutCmd: TAction;
    EditCopyCmd: TAction;
    EditPasteCmd: TAction;
    EditUndoCmd: TAction;
    EditFontCmd: TAction;
    FileSaveAsCmd: TAction;
    ToolButton3: TToolButton;
    QuickPdf1: TQuickPdf;
    GeneratePDF: TAction;
    PDFSaveDialog: TSaveDialog;

    procedure SelectionChange(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure ShowHint(Sender: TObject);
    procedure FileNew(Sender: TObject);
    procedure FileOpen(Sender: TObject);
    procedure FileSave(Sender: TObject);
    procedure FileSaveAs(Sender: TObject);
    procedure FilePrint(Sender: TObject);
    procedure FileExit(Sender: TObject);
    procedure EditUndo(Sender: TObject);
    procedure EditCut(Sender: TObject);
    procedure EditCopy(Sender: TObject);
    procedure EditPaste(Sender: TObject);
    procedure SelectFont(Sender: TObject);
    procedure RulerResize(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure FormPaint(Sender: TObject);
    procedure BoldButtonClick(Sender: TObject);
    procedure ItalicButtonClick(Sender: TObject);
    procedure FontSizeChange(Sender: TObject);
    procedure AlignButtonClick(Sender: TObject);
    procedure FontNameChange(Sender: TObject);
    procedure UnderlineButtonClick(Sender: TObject);
    procedure BulletsButtonClick(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure RulerItemMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure RulerItemMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure FirstIndMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure LeftIndMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure RightIndMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure FormShow(Sender: TObject);
    procedure RichEditChange(Sender: TObject);
    procedure ActionList2Update(Action: TBasicAction;
      var Handled: Boolean);
    procedure GeneratePDFExecute(Sender: TObject);
  private
    FFileName: string;
    FUpdating: Boolean;
    FGenerating: Boolean;
    FDragOfs: Integer;
    FDragging: Boolean;

    TableHeaderFont: TFont;
    HeaderFont: TFont;
    TextFont: TFont;
    BoldTextFont: TFont;
    LinkFont: TFont;

    function CurrText: TTextAttributes;
    procedure GetFontNames;
    procedure SetFileName(const FileName: String);
    procedure CheckFileSave;
    procedure SetupRuler;
    procedure SetEditRect;
    procedure UpdateCursorPos;
    procedure WMDropFiles(var Msg: TWMDropFiles); message WM_DROPFILES;
    procedure PerformFileOpen(const AFileName: string);
    procedure SetModified(Value: Boolean);
    procedure CreateFonts;
    procedure DestroyFonts;
  end;

var
  MainForm: TMainForm;

implementation

uses RichEdit, ShellAPI;

resourcestring
  sSaveChanges = 'Save changes to %s?';
  sOverWrite = 'OK to overwrite %s';
  sUntitled = 'Untitled';
  sModified = 'Modified';
  sColRowInfo = 'Line: %3d   Col: %3d';

const
  RulerAdj = 4/3;
  GutterWid = 6;

  ENGLISH = (SUBLANG_ENGLISH_US shl 10) or LANG_ENGLISH;
  FRENCH  = (SUBLANG_FRENCH shl 10) or LANG_FRENCH;
  GERMAN  = (SUBLANG_GERMAN shl 10) or LANG_GERMAN;

{$R *.DFM}

procedure TMainForm.SelectionChange(Sender: TObject);
begin
  if FGenerating then exit;
  with Editor.Paragraph do
  try
    FUpdating := True;
    FirstInd.Left := Trunc(FirstIndent*RulerAdj)-4+GutterWid;
    LeftInd.Left := Trunc((LeftIndent+FirstIndent)*RulerAdj)-4+GutterWid;
    RightInd.Left := Ruler.ClientWidth-6-Trunc((RightIndent+GutterWid)*RulerAdj);
    BoldButton.Down := fsBold in Editor.SelAttributes.Style;
    ItalicButton.Down := fsItalic in Editor.SelAttributes.Style;
    UnderlineButton.Down := fsUnderline in Editor.SelAttributes.Style;
    BulletsButton.Down := Boolean(Numbering);
    FontSize.Text := IntToStr(Editor.SelAttributes.Size);
    FontName.Text := Editor.SelAttributes.Name;
    case Ord(Alignment) of
      0: LeftAlign.Down := True;
      1: RightAlign.Down := True;
      2: CenterAlign.Down := True;
    end;
    UpdateCursorPos;
  finally
    FUpdating := False;
  end;
end;

function TMainForm.CurrText: TTextAttributes;
begin
  if Editor.SelLength > 0 then Result := Editor.SelAttributes
  else Result := Editor.DefAttributes;
end;

function EnumFontsProc(var LogFont: TLogFont; var TextMetric: TTextMetric;
  FontType: Integer; Data: Pointer): Integer; stdcall;
begin
  TStrings(Data).Add(LogFont.lfFaceName);
  Result := 1;
end;

procedure TMainForm.GetFontNames;
var
  DC: HDC;
begin
  DC := GetDC(0);
  EnumFonts(DC, nil, @EnumFontsProc, Pointer(FontName.Items));
  ReleaseDC(0, DC);
  FontName.Sorted := True;
end;

procedure TMainForm.SetFileName(const FileName: String);
begin
  FFileName := FileName;
  Caption := Format('%s - %s', [ExtractFileName(FileName), Application.Title]);
end;

procedure TMainForm.CheckFileSave;
var
  SaveResp: Integer;
begin
  if not Editor.Modified then Exit;
  SaveResp := MessageDlg(Format(sSaveChanges, [FFileName]),
    mtConfirmation, mbYesNoCancel, 0);
  case SaveResp of
    idYes: FileSave(Self);
    idNo: {Nothing};
    idCancel: Abort;
  end;
end;

procedure TMainForm.SetupRuler;
var
  I: Integer;
  S: String;
begin
  SetLength(S, 201);
  I := 1;
  while I < 200 do
  begin
    S[I] := #9;
    S[I+1] := '|';
    Inc(I, 2);
  end;
  Ruler.Caption := S;
end;

procedure TMainForm.SetEditRect;
var
  R: TRect;
begin
  with Editor do
  begin
    R := Rect(GutterWid, 0, ClientWidth-GutterWid, ClientHeight);
    SendMessage(Handle, EM_SETRECT, 0, Longint(@R));
  end;
end;

{ Event Handlers }

procedure TMainForm.FormCreate(Sender: TObject);
begin
  Application.OnHint := ShowHint;
  OpenDialog.InitialDir := ExtractFilePath(ParamStr(0));
  SaveDialog.InitialDir := OpenDialog.InitialDir;

  SetFileName(sUntitled);
  GetFontNames;
  SetupRuler;
  SelectionChange(Self);

  CurrText.Name := DefFontData.Name;
  CurrText.Size := -MulDiv(DefFontData.Height, 72, Screen.PixelsPerInch);

  Editor.Lines.LoadFromFile('Description.rtf');
  SetFileName('Description.rtf');
end;

procedure TMainForm.ShowHint(Sender: TObject);
begin
  if Length(Application.Hint) > 0 then
  begin
    StatusBar.SimplePanel := True;
    StatusBar.SimpleText := Application.Hint;
  end
  else StatusBar.SimplePanel := False;
end;

procedure TMainForm.FileNew(Sender: TObject);
begin
  SetFileName(sUntitled);
  Editor.Lines.Clear;
  Editor.Modified := False;
  SetModified(False);
end;

procedure TMainForm.PerformFileOpen(const AFileName: string);
begin
  Editor.Lines.LoadFromFile(AFileName);
  SetFileName(AFileName);
  Editor.SetFocus;
  Editor.Modified := False;
  SetModified(False);
end;

procedure TMainForm.FileOpen(Sender: TObject);
begin
  CheckFileSave;
  if OpenDialog.Execute then
  begin
    PerformFileOpen(OpenDialog.FileName);
    Editor.ReadOnly := ofReadOnly in OpenDialog.Options;
  end;
end;

procedure TMainForm.FileSave(Sender: TObject);
begin
  if FFileName = sUntitled then
    FileSaveAs(Sender)
  else
  begin
    Editor.Lines.SaveToFile(FFileName);
    Editor.Modified := False;
    SetModified(False);
  end;
end;

procedure TMainForm.FileSaveAs(Sender: TObject);
begin
  if SaveDialog.Execute then
  begin
    if FileExists(SaveDialog.FileName) then
      if MessageDlg(Format(sOverWrite, [SaveDialog.FileName]),
        mtConfirmation, mbYesNoCancel, 0) <> idYes then Exit;
    Editor.Lines.SaveToFile(SaveDialog.FileName);
    SetFileName(SaveDialog.FileName);
    Editor.Modified := False;
    SetModified(False);
  end;
end;

procedure TMainForm.FilePrint(Sender: TObject);
begin
  if PrintDialog.Execute then
    Editor.Print(FFileName);
end;

procedure TMainForm.FileExit(Sender: TObject);
begin
  Close;
end;

procedure TMainForm.EditUndo(Sender: TObject);
begin
  with Editor do
    if HandleAllocated then SendMessage(Handle, EM_UNDO, 0, 0);
end;

procedure TMainForm.EditCut(Sender: TObject);
begin
  Editor.CutToClipboard;
end;

procedure TMainForm.EditCopy(Sender: TObject);
begin
  Editor.CopyToClipboard;
end;

procedure TMainForm.EditPaste(Sender: TObject);
begin
  Editor.PasteFromClipboard;
end;

procedure TMainForm.SelectFont(Sender: TObject);
begin
  FontDialog1.Font.Assign(Editor.SelAttributes);
  if FontDialog1.Execute then
    CurrText.Assign(FontDialog1.Font);
  SelectionChange(Self);
  Editor.SetFocus;
end;

procedure TMainForm.RulerResize(Sender: TObject);
begin
  RulerLine.Width := Ruler.ClientWidth - (RulerLine.Left*2);
end;

procedure TMainForm.FormResize(Sender: TObject);
begin
  SetEditRect;
  SelectionChange(Sender);
end;

procedure TMainForm.FormPaint(Sender: TObject);
begin
  SetEditRect;
end;

procedure TMainForm.BoldButtonClick(Sender: TObject);
begin
  if FUpdating then Exit;
  if BoldButton.Down then
    CurrText.Style := CurrText.Style + [fsBold]
  else
    CurrText.Style := CurrText.Style - [fsBold];
end;

procedure TMainForm.ItalicButtonClick(Sender: TObject);
begin
  if FUpdating then Exit;
  if ItalicButton.Down then
    CurrText.Style := CurrText.Style + [fsItalic]
  else
    CurrText.Style := CurrText.Style - [fsItalic];
end;

procedure TMainForm.FontSizeChange(Sender: TObject);
begin
  if FUpdating then Exit;
  CurrText.Size := StrToInt(FontSize.Text);
end;

procedure TMainForm.AlignButtonClick(Sender: TObject);
begin
  if FUpdating then Exit;
  Editor.Paragraph.Alignment := TAlignment(TControl(Sender).Tag);
end;

procedure TMainForm.FontNameChange(Sender: TObject);
begin
  if FUpdating then Exit;
  CurrText.Name := FontName.Items[FontName.ItemIndex];
end;

procedure TMainForm.UnderlineButtonClick(Sender: TObject);
begin
  if FUpdating then Exit;
  if UnderlineButton.Down then
    CurrText.Style := CurrText.Style + [fsUnderline]
  else
    CurrText.Style := CurrText.Style - [fsUnderline];
end;

procedure TMainForm.BulletsButtonClick(Sender: TObject);
begin
  if FUpdating then Exit;
  Editor.Paragraph.Numbering := TNumberingStyle(BulletsButton.Down);
end;

procedure TMainForm.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
begin
  try
    CheckFileSave;
  except
    CanClose := False;
  end;
end;

{ Ruler Indent Dragging }

procedure TMainForm.RulerItemMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  FDragOfs := (TLabel(Sender).Width div 2);
  TLabel(Sender).Left := TLabel(Sender).Left+X-FDragOfs;
  FDragging := True;
end;

procedure TMainForm.RulerItemMouseMove(Sender: TObject; Shift: TShiftState;
  X, Y: Integer);
begin
  if FDragging then
    TLabel(Sender).Left :=  TLabel(Sender).Left+X-FDragOfs
end;

procedure TMainForm.FirstIndMouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  FDragging := False;
  Editor.Paragraph.FirstIndent := Trunc((FirstInd.Left+FDragOfs-GutterWid) / RulerAdj);
  LeftIndMouseUp(Sender, Button, Shift, X, Y);
end;

procedure TMainForm.LeftIndMouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  FDragging := False;
  Editor.Paragraph.LeftIndent := Trunc((LeftInd.Left+FDragOfs-GutterWid) / RulerAdj)-Editor.Paragraph.FirstIndent;
  SelectionChange(Sender);
end;

procedure TMainForm.RightIndMouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  FDragging := False;
  Editor.Paragraph.RightIndent := Trunc((Ruler.ClientWidth-RightInd.Left+FDragOfs-2) / RulerAdj)-2*GutterWid;
  SelectionChange(Sender);
end;

procedure TMainForm.UpdateCursorPos;
var
  CharPos: TPoint;
begin
  CharPos.Y := SendMessage(Editor.Handle, EM_EXLINEFROMCHAR, 0,
    Editor.SelStart);
  CharPos.X := (Editor.SelStart -
    SendMessage(Editor.Handle, EM_LINEINDEX, CharPos.Y, 0));
  Inc(CharPos.Y);
  Inc(CharPos.X);
  StatusBar.Panels[0].Text := Format(sColRowInfo, [CharPos.Y, CharPos.X]);
end;

procedure TMainForm.FormShow(Sender: TObject);
begin
  UpdateCursorPos;
  DragAcceptFiles(Handle, True);
  RichEditChange(nil);
  Editor.SetFocus;
  { Check if we should load a file from the command line }
  if (ParamCount > 0) and FileExists(ParamStr(1)) then
    PerformFileOpen(ParamStr(1));
end;

procedure TMainForm.WMDropFiles(var Msg: TWMDropFiles);
var
  CFileName: array[0..MAX_PATH] of Char;
begin
  try
    if DragQueryFile(Msg.Drop, 0, CFileName, MAX_PATH) > 0 then
    begin
      CheckFileSave;
      PerformFileOpen(CFileName);
      Msg.Result := 0;
    end;
  finally
    DragFinish(Msg.Drop);
  end;
end;

procedure TMainForm.RichEditChange(Sender: TObject);
begin
  SetModified(Editor.Modified);
end;

procedure TMainForm.SetModified(Value: Boolean);
begin
  if Value then StatusBar.Panels[1].Text := sModified
  else StatusBar.Panels[1].Text := '';
end;

procedure TMainForm.ActionList2Update(Action: TBasicAction;
  var Handled: Boolean);
begin
 { Update the status of the edit commands }
  EditCutCmd.Enabled := Editor.SelLength > 0;
  EditCopyCmd.Enabled := EditCutCmd.Enabled;
  if Editor.HandleAllocated then
  begin
    EditUndoCmd.Enabled := Editor.Perform(EM_CANUNDO, 0, 0) <> 0;
    EditPasteCmd.Enabled := Editor.Perform(EM_CANPASTE, 0, 0) <> 0;
  end;
end;

procedure TMainForm.CreateFonts;
begin
  // Creating a font for a table header
  TableHeaderFont := TFont.Create();
  with TableHeaderFont do
    begin
      Color := clBlack;
      Size := 12;
      Name := 'Impact';
    end;

  // Creating a font for a simple text
  TextFont := TFont.Create();
  with TextFont do
    begin
      Color := clBlack;
      Size := 12;
      Name := 'Times New Roman';
    end;

  // Creating a font for a link text
  LinkFont := TFont.Create();
  with LinkFont do
    begin
      Color := clBlue;
      Size := 12;
      Name := 'Times New Roman';
    end;

  // Creating a font for a bold text
  BoldTextFont := TFont.Create();
  with BoldTextFont do
    begin
      Color := clBlack;
      Style := Style + [fsBold];
      Size := 10;
      Name := 'Times New Roman';
    end;

  // Creating a font for a document header
  HeaderFont := TFont.Create();
  with HeaderFont do
    begin
      Color := clNavy;
      Size := 12;
      Style := Style + [fsBold];
      Name := 'Times New Roman';
    end;
end;

procedure TMainForm.DestroyFonts;
begin
  TableHeaderFont.Free;
  HeaderFont.Free;
  TextFont.Free;
  BoldTextFont.Free;
  LinkFont.Free;
end;

procedure TMainForm.GeneratePDFExecute(Sender: TObject);
  function CompareAttributes(First: TFont ; Second: TTextAttributes) : boolean;
  begin
    CompareAttributes := false;
    with First do
    begin
      If CharSet <> Second.Charset then Exit;
      If Color <> Second.Color then Exit;
      If Name <> Second.Name then Exit;
      If Pitch <> Second.Pitch then Exit;
      If Size <> Second.Size then Exit;
      If Style <> Second.Style then Exit;
      CompareAttributes := true;
    end;
  end;
  procedure AssignAttributes(Source: TTextAttributes; var Dest: TFont);
  begin
    with Dest do
    begin
      CharSet := Source.Charset;
      Color := Source.Color;
      Name := Source.Name;
      Pitch := Source.Pitch;
      Size := Source.Size;
      Style := Source.Style;
    end;
  end;
var i: integer;
    Alignment: TAlignment;
    TextAlign: TRenderTextAlign;
    Line: String;
    Attributes: TFont;
    Caret: TPoint;
    HeaderTable, PricingTable: TRenderTable;
    LeftPicture,
    RightPicture: TPicture;
    AParagraph: TRenderCell;
begin
  Alignment := taLeftJustify;
  TextALign := rtaLeft;

  // Receiving a file name
  if  PDFSaveDialog.Execute then
    QuickPDF1.FileName := PDFSaveDialog.FileName
  else exit;

  // Loading pictures
  LeftPicture := TPicture.Create();
  LeftPicture.LoadFromFile(ExtractFilePath(Application.ExeName) +
                           '\logo-ems.jpg');
  RightPicture := TPicture.Create();
  RightPicture.LoadFromFile(ExtractFilePath(Application.ExeName) +
                            '\slogan.bmp');

  // Defining the document padding
  QuickPDF1.Document.MainTable.Padding.Left := 28;
  QuickPDF1.Document.MainTable.Padding.Right := 28;

  CreateFonts;
  try
    // Creating a document header
    HeaderTable := TRenderTable.Create(
        [(QuickPDF1.Document.Section[0].InternalWidth) div 2,
         (QuickPDF1.Document.Section[0].InternalWidth) div 2]);
    with HeaderTable do begin
      Border := -1;
      AddRow;
      Cell[0, 0].AddItem(0, 0, TRenderImage.Create(LeftPicture.Graphic));
      Cell[1, 0].AddText('', TextFont, rtaRight);
      Cell[1, 0].AddTextItem(TRenderImage.Create(RightPicture.Graphic));
    end;
    QuickPDF1.Document.Header.Additem(0, 0, HeaderTable);

    // Adding a bookmark to the document header
    QuickPDF1.Document.Header.LinkName := 'Header';
    QuickPDF1.Contents.Add('Header', 'Header');

    i := 0;
    Editor.Enabled := false;
    Caret := Editor.CaretPos;
    Attributes := TFont.Create();
    AssignAttributes(Editor.SelAttributes, Attributes);

    // Adding a bookmark to the beginning of the document
    AParagraph := QuickPDF1.Document.Section[0].AddParagraph;
    AParagraph.LinkName := 'doc';
    QuickPDF1.Contents.Add('Main Text', 'doc');

    while i < Length(Editor.Lines.Text) do begin
      // Selecting a symbol
      Editor.SelStart := i;
      Editor.SelLength := 1;

      if (Caret.y <> Editor.CaretPos.y) then begin
        // Inserting a line break
        Line := Line + #10;
        Caret := Editor.CaretPos;
      end;

      // Comparing attributes of the selected symbol with attributes of
      // the previous symbol
      if (CompareAttributes(Attributes, Editor.SelAttributes) and
          (Alignment = Editor.Paragraph.Alignment) and
          (AParagraph.Indent = Editor.Paragraph.FirstIndent) and
          (AParagraph.Padding.Left = Editor.Paragraph.LeftIndent) and
          (AParagraph.Padding.Right = -Editor.Paragraph.RightIndent)) then
      begin
        Line := Line + Editor.SelText;
        inc(i);
      end
      else begin
        // Writing a text to the document
        AParagraph.AddText(Line,Attributes,TextAlign);
        Line := '';

        // Comparing attributes of the current paragraph with attributes of
        // the previous paragraph
        if ((Alignment <> Editor.Paragraph.Alignment) or
            (AParagraph.Indent <> Editor.Paragraph.FirstIndent) or
            (AParagraph.Padding.Left <> Editor.Paragraph.LeftIndent) or
            (AParagraph.Padding.Right <> -Editor.Paragraph.RightIndent)) then
        begin
          Alignment := Editor.Paragraph.Alignment;
          // Adding a new paragraph with new attributes
          AParagraph := QuickPDF1.Document.Section[0].AddParagraph;
          with AParagraph do begin
            Padding.Left := Editor.Paragraph.LeftIndent;
            Padding.Right := -Editor.Paragraph.RightIndent;
            Indent := Editor.Paragraph.FirstIndent;
          end;
          case Alignment of
            taLeftJustify: TextAlign := rtaLeft;
            taRightJustify: TextAlign :=rtaRight;
            taCenter: TextAlign := rtaCenter;
          end;
        end;
        if not CompareAttributes(Attributes, Editor.SelAttributes)
          then AssignAttributes(Editor.SelAttributes, Attributes);
      end;
    end;
    QuickPDF1.Document.Section[0].AddText(Line,Attributes,TextAlign);

    // Adding a pricing information
    AParagraph := QuickPDF1.Document.Section[0].AddParagraph;
    AParagraph.LinkName := 'table';
    QuickPDF1.Contents.Add('Pricing Information', 'table');
    AParagraph.Padding.Left := 36;
    AParagraph.AddText(#10'Pricing Information'#10#10, HeaderFont, rtaLeft);
    AParagraph := QuickPDF1.Document.Section[0].AddParagraph;
    PricingTable := TRenderTable.Create([
        (QuickPDF1.Document.Section[0].InternalWidth) div 2,
        (QuickPDF1.Document.Section[0].InternalWidth) div 4,
        (QuickPDF1.Document.Section[0].InternalWidth) div 4]);
    with PricingTable do begin
      Padding := Rect(2, 2, 2, 2);
      AddRow;
      Rows[0].BgColor := clSilver;
      Cell[0, 0].AddText('Product Description', TableHeaderFont, rtaLeft);
      Cell[1, 0].AddText('Price', TableHeaderFont, rtaCenter);
      Cell[0,1].AddText('EMS QuickPDF', TextFont, rtaLeft);
      Cell[1,1].AddText('$195', TextFont, rtaCenter);
      Cell[2,1].AddText('Register Now!', LinkFont, rtaCenter,
        'http://www.ems-hitech.com/quickpdf/purchase.phtml');
    end;
    AParagraph.AddItem(0, 0, PricingTable);

    // Adding the last paragraph of the document
    AParagraph := QuickPDF1.Document.Section[0].AddParagraph;
    AParagraph.AddText(#10+'You can also purchase EMS QuickPDF in a bundle '
        + 'along with several other our products and save a lot of your '
        + 'money!'#10#10, TextFont, rtaLeft);
    AParagraph.AddText('For more information please visit our site at ',
        TextFont, rtaLeft);
    AParagraph.AddText('http://www.ems-hitech.com/quickpdf/'#10#10, LinkFont,
        rtaLeft, 'http://www.ems-hitech.com/quickpdf/');

    // Generating a document
    QuickPDF1.GenerateToFile;

    // Opening the result document
    ShellExecute(0, 'open', PChar(QuickPDF1.FileName), '', '', SW_SHOWNORMAL);
  finally
    DestroyFonts;
    Editor.Enabled := true;
  end;
end;

end.
